@@ -1,5 +1,5 @@
 from datetime import datetime
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 from rest_framework.exceptions import ValidationError
 import pytz
 from dateutil.parser import parse as dateutil_parse
@@ -33,13 +33,12 @@ def parse_bool(v, default = None, exceptions: bool = True) -> bool:
     s = str(v).lower()
     if s in TRUE_VALUES:
         return True
-    elif s in FALSE_VALUES:
+    if s in FALSE_VALUES:
         return False
-    else:
-        if exceptions:
-            msg = _("%(value)s is not one of the available choices") % {'value': v}
-            raise ValidationError(msg)
-        return default
+    if exceptions:
+        msg = _("%(value)s is not one of the available choices") % {'value': v}
+        raise ValidationError(msg)
+    return default
 
 
 def parse_datetime(v: str, default = None, tz = None, exceptions: bool = True) -> datetime:
@@ -58,6 +57,6 @@ def parse_datetime(v: str, default = None, tz = None, exceptions: bool = True) -
         return t if t.tzinfo else tz.localize(t)
     except Exception:
         if exceptions:
-            msg = _("'%(value)s' value has an invalid format. It must be in YYYY-MM-DD HH:MM[:ss[.uuuuuu]][TZ] format.") % {'value': v}
+            msg = _("“%(value)s” value has an invalid format. It must be in YYYY-MM-DD HH:MM[:ss[.uuuuuu]][TZ] format.") % {'value': v}
             raise ValidationError(msg)
         return default
